#ifndef __Corrector_h__
#define __Corrector_h__

#include "BThread.h"
#include "IEvent.h"
#include "String.h"
#include "T/Truth.h"

class Corrector : public BThread
{
public:
    Corrector() : BThread( "Corrector" )
    {
    }

    void entryPoint()
    {
        while ( T::alwaysTrue() )
        {
            waitForAllEvents();

            if ( lastEvent() == IEvent::TEMPERATURE_HIGH )
            {
                decreaseTemperature();
            }
            else if ( lastEvent() == IEvent::TEMPERATURE_LOW )
            {
                increaseTemperature();
            }
            else if ( lastEvent() == IEvent::HUMIDITY_HIGH )
            {
                decreaseHumidity();
            }
            else if ( lastEvent() == IEvent::HUMIDITY_LOW )
            {
                increaseHumidity();
            }
        }
    }

private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    void waitForAllEvents()
    {
        clearAll();
        _watched = IEvent::allEvents();
        bSync( _requested, _watched, _blocked, "Waiting for requests" );
    }

    void increaseTemperature()
    {
        printf( "\t\t>> Increase temperature\n" );
        issueTemperatureOkNotice();
    }

    void decreaseTemperature()
    {
        printf( "\t\t>> Decrease temperature\n" );
        issueTemperatureOkNotice();
    }

    void increaseHumidity()
    {
        printf( "\t\t>> Increase humidity\n" );
        issueHumidityOkNotice();
    }

    void decreaseHumidity()
    {
        printf( "\t\t>> Decrease humidity\n" );
        issueHumidityOkNotice();
    }

    void issueTemperatureOkNotice()
    {
        clearAll();
        _requested.append( IEvent::TEMPERATURE_ADJUSTED );
        bSync( _requested, _watched, _blocked, "Temperature adjusted" );
    }

    void issueHumidityOkNotice()
    {
        clearAll();
        _requested.append( IEvent::HUMIDITY_ADJUSTED );
        bSync( _requested, _watched, _blocked, "Humidity adjusted" );
    }
};

#endif // __Corrector_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
