#ifndef __MaintenanceTimer_h__
#define __MaintenanceTimer_h__

#include "BThread.h"
#include "IEvent.h"
#include "String.h"
#include "T/Truth.h"

class MaintenanceTimer : public BThread
{
public:
    MaintenanceTimer() : BThread( "MaintenanceTimer" ), _maintenanceCount( 0 )
    {
    }

    void entryPoint()
    {
        enum {
            ONE_TENTH_OF_A_SECOND = 100000,
        };

        _lastTime = now();

        while ( T::alwaysTrue() )
        {
            while ( !timeForMaintenance() )
            {
                usleep( ONE_TENTH_OF_A_SECOND );

                // clearAll();
                // _watched = IEvent::allEvents();
                // bSync( _requested, _watched, _blocked, "Idle" );
            }

            requestMaintenance();
        }
    }

private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    time_t _lastTime;

    unsigned _maintenanceCount;

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    static time_t now()
    {
        return time( NULL );
    }

    bool timeForMaintenance() const
    {
        enum {
            THRESHOLD_IN_SECONDS = 15,
        };

        return ( now() - _lastTime ) >= THRESHOLD_IN_SECONDS;
    }

    void requestMaintenance()
    {
        clearAll();
        _requested.append( IEvent::INITIATE_MAINTENANCE );

        bSync( _requested, _watched, _blocked, "Requesting" );
        _lastTime = now();

        ++_maintenanceCount;

        printf( "Maintenance count: %u\n", _maintenanceCount );
    }
};

#endif // __MaintenanceTimer_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
