#include "AutomaticKeyPad.h"
#include "BProgram.h"
#include "Corrector.h"
#include "Dispenser.h"
#include "IEvent.h"
#include "KeyPad.h"
#include "MaintenanceTimer.h"
#include "Measurer.h"
#include "MeasurerStateGraph.h"
#include "Speaker.h"
#include <cstdio>
#include <cstdlib>

int main()
{
    srand( time( NULL ) );

    try
    {
        BProgram bProgram;

        Vector<Event> blockables;
        bProgram.addThreadBlockingData( "KeyPad", blockables );
        bProgram.addThreadBlockingData( "Dispenser", blockables );
        bProgram.addThreadBlockingData( "MaintenanceTimer", blockables );
        bProgram.addThreadBlockingData( "Speaker", blockables );
        bProgram.addThreadBlockingData( "Corrector", blockables );

        // blockables.append( IEvent::PRODUCT_PROVIDED );
        // bProgram.addThreadBlockingData( "Measurer", blockables );

        ThreadStateGraph measurerStateGraph = MeasurerStateGraph::stateGraph();
        bProgram.addThreadStateGraph( "Measurer", measurerStateGraph );

        bProgram.runEngine();

        sleep( 1 );

        MaintenanceTimer maintenanceTimer;
        KeyPad keyPad;
        //AutomaticKeyPad keyPad;
        Dispenser dispenser;
        Measurer measurer;
        Corrector corrector;
        Speaker speaker;

        sleep( 1 );

        bProgram.addThread( maintenanceTimer );
        bProgram.addThread( keyPad );
        bProgram.addThread( dispenser );
        bProgram.addThread( measurer );
        bProgram.addThread( corrector );
        bProgram.addThread( speaker );

        sleep( 1 );

        bProgram.runProgram();

        sleep( 10000 );
    }
    catch ( const Error &e )
    {
        printf( "Main Thread: Error caught. Code: %u. Errno: %i\n", e.code(), e.errno() );
        fflush( 0 );
    }

	return 0;
}

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
